@@ -4,6 +4,7 @@ from scipy import stats
 from math import sqrt
 from sklearn import linear_model
 from sklearn.ensemble import RandomForestRegressor
+from sklearn.model_selection import GridSearchCV
 from sklearn.metrics import mean_squared_error
 from geosoup.common import Handler, Opt
 from geosoup.raster import Raster, np, gdal_array, gdal
@@ -27,6 +28,7 @@ class _Regressor(object):
                  data=None,
                  regressor=None,
                  **kwargs):
+
         self.data = data
         self.vdata = None
         self.regressor = regressor
@@ -37,9 +39,20 @@ class _Regressor(object):
         self.training_results = dict()
         self.fit = False
         self.all_cv_results = None
+        self.cv_param = None
 
         self.adjustment = dict()
 
+        if hasattr(self.regressor, 'intercept_'):
+            self.intercept = self.regressor.intercept_
+        else:
+            self.intercept = None
+
+        if hasattr(self.regressor, 'coef_'):
+            self.coefficient = self.regressor.coef_
+        else:
+            self.coefficient = None
+
         if kwargs is not None:
             if 'timer' in kwargs:
                 _Regressor.time_it = kwargs['timer']
@@ -69,18 +82,6 @@ class _Regressor(object):
         self.label = data['label_name']
         self.fit = True
 
-        if hasattr(self, 'intercept'):
-            if hasattr(self.regressor, 'intercept_'):
-                self.intercept = self.regressor.intercept_
-
-        if hasattr(self, 'coefficient'):
-            if hasattr(self.regressor, 'coef_'):
-                self.coefficient = self.regressor.coef_
-
-    def predict(self, *args, **kwargs):
-        """Placeholder function"""
-        return
-
     def pickle_it(self,
                   outfile):
         """
@@ -91,9 +92,8 @@ class _Regressor(object):
         with open(outfile, 'wb') as fileptr:
             pickle.dump(self, fileptr)
 
-    @classmethod
-    def load_from_pickle(cls,
-                         infile):
+    @staticmethod
+    def load_from_pickle(infile):
         """
         Reload regressor from file
         :param infile: File to load regressor from
@@ -102,6 +102,46 @@ class _Regressor(object):
             regressor_obj = pickle.load(fileptr)
             return regressor_obj
 
+    def grid_search_param(self,
+                          param_dict,
+                          scoring='score',
+                          cv_folds=5,
+                          n_jobs=1,
+                          allowed_grad=0.1,
+                          select_perc=90):
+        """
+        Method to search optimal parameters for the regressor
+        :param param_dict:
+        :param scoring:
+        :param cv_folds:
+        :param n_jobs:
+        :param allowed_grad:
+        :param select_perc:
+        """
+        model = GridSearchCV(self.regressor,
+                             param_dict,
+                             scoring=scoring,
+                             cv=cv_folds,
+                             n_jobs=n_jobs)
+
+        results = model.cv_results_
+
+        params = results['params']
+        mean_scores = results['mean_test_score']
+        std_scores = results['std_test_score']
+        ranks = results['rank_test_scores']
+
+        grad_cond = (std_scores.max() - std_scores.min())/std_scores.max() <= allowed_grad
+
+        if not grad_cond:
+            param = params[np.where(mean_scores == np.percentile(mean_scores,
+                                                                 select_perc,
+                                                                 interpolation='nearest'))]
+        else:
+            param = params[np.where(ranks == 1)]
+
+        self.cv_param = param
+
     @staticmethod
     def get_defaults(**kwargs):
         """
