@@ -515,7 +515,7 @@ class Samples:
         fold_samples = list()
 
         for fold_samp in nsamp_list:
-            if index_list.size[0] < (2 * fold_samp):
+            if index_list.shape[0] < (2 * fold_samp):
                 val_index = np.random.choice(index_list,
                                              size=fold_samp,
                                              replace=False)
